//============================================
// ESP8266 Web Server
// FB 2023-06-15
//============================================
#include <ESP8266WiFi.h>
#include <ESPAsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include "webpage.h"

// credentials
const char *ssid = "";
const char *password = "";

// web server
AsyncWebServer server(80);

// GPIO pins
const int GPIO_EYE_ENABLE = 12;
const int GPIO_MAIN_FUNCT = 13;
const int GPIO_SEAT_FUNCT = 4;
const int GPIO_SEAT_UPDWN = 5;

// button ID
enum buttons {
  BUTTON_EYE_ENABLE = 1,
  BUTTON_MAIN_FUNCT,
  BUTTON_SEAT_FUNCT,
  BUTTON_SEAT_UPDN1,
  BUTTON_SEAT_UPDN2,
  BUTTON_SEAT_UPDN3,
  BUTTON_SEAT_UPDN4,
  BUTTON_SEAT_UPDN5,
};

//------------------------------------------
void setup()
{
  Serial.begin(115200);

  // set GPIO output to low
  digitalWrite(GPIO_EYE_ENABLE, LOW);
  digitalWrite(GPIO_MAIN_FUNCT, LOW);
  digitalWrite(GPIO_SEAT_FUNCT, LOW);
  digitalWrite(GPIO_SEAT_UPDWN, LOW);

  // config GPIOs as output
  pinMode(GPIO_EYE_ENABLE, OUTPUT);
  pinMode(GPIO_MAIN_FUNCT, OUTPUT);
  pinMode(GPIO_SEAT_FUNCT, OUTPUT);
  pinMode(GPIO_SEAT_UPDWN, OUTPUT);

  // Connect to Wi-Fi
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }
  // print local IP address and start web server
  Serial.println();
  Serial.println("WiFi connected.");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  // Route for root / web page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request)
  { 
    request->send_P(200, "text/html", index_html); 
  });

  // Send a GET request to <ESP_IP>/update?output=<val_output>&state=<val_status>
  server.on("/update", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    String val_output = "na";
    if (request->hasParam("output")) 
    {
      val_output = request->getParam("output")->value();
      Serial.println("GPIO" + String(val_output));
      
      switch(val_output.toInt())
      {
        case BUTTON_EYE_ENABLE: 
          digitalWrite(GPIO_EYE_ENABLE, HIGH); 
          delay(250);
          digitalWrite(GPIO_EYE_ENABLE, LOW); 
          break;
        case BUTTON_MAIN_FUNCT: 
          digitalWrite(GPIO_MAIN_FUNCT, HIGH); 
          delay(250);
          digitalWrite(GPIO_MAIN_FUNCT, LOW); 
          break;
        case BUTTON_SEAT_FUNCT: 
          digitalWrite(GPIO_SEAT_FUNCT, HIGH); 
          delay(250);
          digitalWrite(GPIO_SEAT_FUNCT, LOW); 
          break;
        case BUTTON_SEAT_UPDN1: 
          digitalWrite(GPIO_SEAT_UPDWN, HIGH); 
          delay(250);
          digitalWrite(GPIO_SEAT_UPDWN, LOW); 
          break;
        case BUTTON_SEAT_UPDN2: 
          digitalWrite(GPIO_SEAT_UPDWN, HIGH); 
          delay(1000);
          digitalWrite(GPIO_SEAT_UPDWN, LOW); 
          break;
        case BUTTON_SEAT_UPDN3: 
          digitalWrite(GPIO_SEAT_UPDWN, HIGH); 
          wdt_disable(); // disable WDT
          delay(2000);
          wdt_enable(WDTO_2S); // enable WDT with a timeout of 2 seconds
          digitalWrite(GPIO_SEAT_UPDWN, LOW); 
          break;
        case BUTTON_SEAT_UPDN4: 
          digitalWrite(GPIO_SEAT_UPDWN, HIGH); 
          wdt_disable(); // disable WDT
          delay(4000);
          wdt_enable(WDTO_2S); // enable WDT with a timeout of 2 seconds
          digitalWrite(GPIO_SEAT_UPDWN, LOW); 
          break;
        case BUTTON_SEAT_UPDN5: 
          digitalWrite(GPIO_SEAT_UPDWN, HIGH); 
          wdt_disable(); // disable WDT
          delay(8000);
          wdt_enable(WDTO_2S); // enable WDT with a timeout of 2 seconds
          digitalWrite(GPIO_SEAT_UPDWN, LOW); 
          break;
      }
    }
    request->send(200, "text/plain", "OK"); 
  });

  // Start server
  server.begin();
}

//------------------------------------------
void loop()
{
}
